#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pymklist` package."""
import os
import zipfile

import pytest

from click.testing import CliRunner

from mklist import cli

import mock


@pytest.fixture
def runner():
    return CliRunner()


@pytest.fixture
def cwd_mock():
    with mock.patch('os.getcwd') as ctx:
        yield ctx


@pytest.fixture
def exists_mock():
    with mock.patch('os.path.exists') as ctx:
        yield ctx


def test_cli_not_valid_ldraw(cwd_mock, exists_mock, runner):
    cwd_mock.side_effect = lambda: 'invalid_dir'
    exists_mock.side_effect = lambda s: False

    result = runner.invoke(cli.main)
    assert result.exit_code == 1
    assert 'mklist' in result.output


def glob_side_effect(*args, **kwargs):
    return []


@mock.patch('mklist.cli.generate_parts_lst')
@mock.patch('glob.glob', side_effect=glob_side_effect)
def test_cli_in_valid_ldraw_dir(glob_mock, generate_parts_lst_mock,
                                cwd_mock, exists_mock, runner):
    cwd_mock.side_effect = lambda: 'ldraw'

    def exists_mock_side_effect(s):
        if s == 'parts.lst':
            return False
        return True

    exists_mock.side_effect = exists_mock_side_effect

    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert not generate_parts_lst_mock.called

    result = runner.invoke(cli.main, ['--description'])
    assert result.exit_code == 0
    assert generate_parts_lst_mock.called

    result = runner.invoke(cli.main, ['--number'])
    assert result.exit_code == 0
    assert generate_parts_lst_mock.called


def test_cli_in_ldraw_dir_without_parts_dir_no(cwd_mock,
                                               exists_mock,
                                               runner):
    cwd_mock.side_effect = lambda: 'ldraw'
    exists_mock.side_effect = lambda s: False

    result = runner.invoke(cli.main)
    assert result.exit_code == 1


def mocked_retrieve(*args, **kwargs):
    return os.path.join('tests', 'test_data', 'complete.zip'), ''


def test_cli_has_help(runner):
    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert '--help' in help_result.output
    assert 'Show this message and exit' in help_result.output



# -*- coding: utf-8 -*-

"""Console script for pymklist."""
import sys
import click
import os

import re

from mklist.generate import generate_parts_lst

ldraw = re.compile(r'ldraw', flags=re.IGNORECASE)
yes = re.compile(r'[y](?:es)?', flags=re.IGNORECASE)


@click.command(name='mklist')
@click.argument('input_directory', required=False)
@click.option('--description', default=False, is_flag=True)
@click.option('--number', default=False, is_flag=True)
def main(input_directory, description, number, ):
    """Console script for pymklist."""
    if input_directory is None:
        input_directory = os.getcwd()
    if re.match(ldraw, input_directory) and os.path.exists('parts'):
        print('operating from a LDraw folder, continuing...')
    elif not (
        re.match(ldraw, input_directory)
        and
        os.path.exists(os.path.join(input_directory, 'parts'))
    ):
        print('LDraw parts directory not found')
        print('Please specify a LDraw parts library directory'
              'location in the arguments of the mklist call')
        raise click.Abort()

    parts_lst_path = os.path.join(input_directory, 'parts.lst')
    parts_folder_path = os.path.join(input_directory, 'parts')
    if description:
        generate_parts_lst('description', parts_folder_path, parts_lst_path)
    if number:
        generate_parts_lst('number', parts_folder_path, parts_lst_path)


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



import os
import sys
import tempfile

import pytest
import filecmp
import difflib

from mklist.generate import generate_parts_lst


@pytest.mark.parametrize('mode', ['description'])
def test_generate_test_data(mode):
    _, parts_file = tempfile.mkstemp(suffix='.lst')
    parts_folder_path = os.path.join('tests', 'test_data',
                                     'ldraw', 'parts')
    generate_parts_lst(mode,
                       parts_folder_path=parts_folder_path,
                       parts_lst_path=parts_file
                       )
    expected_parts_file = os.path.join('tests',
                                       'test_data',
                                       'ldraw',
                                       'parts.%s.lst' % mode)

    comparison = filecmp.cmp(parts_file,
                             expected_parts_file,
                             shallow=False)

    if not comparison:
        content = open(parts_file).readlines()
        expected = open(expected_parts_file).readlines()
        for line in difflib.context_diff(expected,
                                         content,
                                         fromfile='expected',
                                         tofile='content'):
            sys.stdout.write(line)

    assert comparison



import glob
import os
import io
import shutil

import re

FORMAT_STRING = '{filename:<30} {description}'
alphanum = re.compile(r"[\W_]+", re.UNICODE)
num = re.compile(r"\D", re.UNICODE)


def line_format(**kwargs):
    return FORMAT_STRING.format(**kwargs)


def do_sort(li, mode):
    def cmpkey1(row):
        return alphanum.sub('', row[mode]).lower()

    def cmpkey2(row):
        return line_format(**row)

    li.sort(key=cmpkey2)
    li.sort(key=cmpkey1)


def get_parts_lst(parts_dir, mode):
    parts = glob.glob(os.path.join(parts_dir, '*.dat'))

    parts_dict = {'_': [], '~': []}
    parts_lst = []

    for part in parts:
        filename = os.path.basename(part)
        number, _ = os.path.splitext(filename)
        try:
            with open(part, 'rU', encoding='utf-8') as part_file:
                header = part_file.readline()
                header_description = header[2:]
                if '~Moved' in header:
                    continue
                row = {'filename': filename,
                       'number': number,
                       'description': header_description}

                if '_' in header_description:
                    parts_dict['_'].append(row)
                elif '~' in header_description:
                    parts_dict['~'].append(row)
                else:
                    parts_lst.append(row)
        except UnicodeDecodeError:
            # ignore non-utf-8 files
            continue

    do_sort(parts_lst, mode)
    do_sort(parts_dict['_'], mode)
    do_sort(parts_dict['~'], mode)

    parts_lst.extend(parts_dict['_'])
    parts_lst.extend(parts_dict['~'])

    return parts_lst


def generate_parts_lst(mode, parts_folder_path, parts_lst_path):
    if os.path.exists(parts_lst_path):
        shutil.move(parts_lst_path, parts_lst_path+'.old')

    parts_lst = get_parts_lst(parts_folder_path, mode)

    lines = [line_format(**row) for row in parts_lst]
    open(parts_lst_path, 'w', newline='\r\n', encoding='utf-8').writelines(lines)
